/* PBSortedComponents.m created by luke on Thu 22-May-1997 */

#import "PBSortedComponents.h"
#import "NSMutableArrayExtensions.h"

#define USER_INFO_DICTIONARY_FILE_CATEGORY_KEY @"key"

static PBSortedComponents *sharedPBSortedComponentsObject = nil;

@implementation PBSortedComponents

+ (void)initialize
{
    [super initialize];

    if (sharedPBSortedComponentsObject)
        return;
    sharedPBSortedComponentsObject = [[self alloc] init];
}

- init
{
    NSNotificationCenter *notificationCenter;
    notificationCenter = [NSNotificationCenter defaultCenter];

    // Handle PBProjectDidOpenNotifications.
    [notificationCenter addObserver:self selector:@selector(projectDidOpen:)
                               name:PBProjectDidOpenNotification
                             object:nil];

    // Handle PBFileAddedToProjectNotifications.
    [notificationCenter addObserver:self selector:@selector(fileAddedToProject:)
                               name:PBFileAddedToProjectNotification
                             object:nil];

    // Handle PBFileRemovedFromProjectNotifications.
    [notificationCenter addObserver:self selector:@selector(fileRemovedFromProject:)
                               name:PBFileRemovedFromProjectNotification
                             object:nil];

    [super init];
    return self;
}

// Project component sorting methods.

- (void)sortProjectComponents:(PBProject *)project
{
    BOOL didSort = NO;

    // Sort all the appropriate, sortable keys provided by PBProject.h

    didSort |= [[project fileListForKey:PB_ClassesKey create:NO] sortIfUnsorted];
    didSort |= [[project fileListForKey:PB_HeadersKey create:NO] sortIfUnsorted];
    didSort |= [[project fileListForKey:PB_OtherSourcesKey create:NO] sortIfUnsorted];
    didSort |= [[project fileListForKey:PB_OtherResourcesKey create:NO] sortIfUnsorted];
    didSort |= [[project fileListForKey:PB_SupportingFilesKey create:NO] sortIfUnsorted];
    didSort |= [[project fileListForKey:PB_SubprojectsKey create:NO] sortIfUnsorted];
    didSort |= [[project fileListForKey:PB_InterfacesKey create:NO] sortIfUnsorted];
    didSort |= [[project fileListForKey:PB_ImagesKey create:NO] sortIfUnsorted];
    didSort |= [[project fileListForKey:PB_LocalizableFilesKey create:NO] sortIfUnsorted];
    didSort |= [[project fileListForKey:PB_FrameworksKey create:NO] sortIfUnsorted];
    didSort |= [[project fileListForKey:PB_OtherLibsKey create:NO] sortIfUnsorted];
    didSort |= [[project fileListForKey:PB_NonProjectFilesKey create:NO] sortIfUnsorted];

    didSort |= [[project fileListForKey:PB_WebComponentsKey create:NO] sortIfUnsorted];
    didSort |= [[project fileListForKey:PB_WebServerResourcesKey create:NO] sortIfUnsorted];
    didSort |= [[project fileListForKey:PB_WOAppResourcesKey create:NO] sortIfUnsorted];

    if (didSort)
        [project setTouched:TOUCHED_PB_PROJECT];
}

- (void)sortProjectComponents:(PBProject *)project withFileKey:(NSString *)fileKey
{
    NSMutableArray *fileList;

    fileList = [project fileListForKey:fileKey];
    [fileList sortUsingSelector:@selector(caseInsensitiveCompare:)];

    [project setTouched:TOUCHED_PB_PROJECT];
}

// Notification handlers.

- (void)projectDidOpen:(NSNotification *)notification
{
    PBProject *project;

    project = (PBProject *)[notification object];
    [self sortProjectComponents:project];
}

- (void)fileAddedToProject:(NSNotification *)notification
{
    [self filesChangedInProject:notification];
}

- (void)fileRemovedFromProject:(NSNotification *)notification
{
    [self filesChangedInProject:notification];
}

- (void)filesChangedInProject:(NSNotification *)notification
{
    PBProject *project;
    NSString  *fileKey;

    project = (PBProject *)[notification object];
    fileKey = [[notification userInfo] objectForKey:USER_INFO_DICTIONARY_FILE_CATEGORY_KEY];

    [self sortProjectComponents:project withFileKey:fileKey];

    // if they just added a WOComponent, ProjectBuilder only sends fileAdded notification for the actual
    // component. since other files may also have been added, we also sort classes, headers and web resources.
    if ([fileKey isEqualToString:PB_WebComponentsKey]) {
        [self sortProjectComponents:project withFileKey:PB_ClassesKey];
        [self sortProjectComponents:project withFileKey:PB_HeadersKey];
        [self sortProjectComponents:project withFileKey:PB_WebServerResourcesKey];
        [self sortProjectComponents:project withFileKey:PB_WOAppResourcesKey];
    }
}

@end
